﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using DacIEServiceCli.IEService;

namespace DacIEServiceCli
{
    public partial class Program
    {
        public void ExportAction()
        {
            Stopwatch sw = new Stopwatch();

            Console.WriteLine("Export started: {0}", DateTime.Now);

            try
            {
                sw.Start();

                IDACWebService channel = this.CreateWebServiceChannel();
                using (channel as IDisposable)
                {
                    var exportInput = this.GetExportInput();
                    Guid resultGuid = channel.Export(exportInput);

                    Console.WriteLine("Operation Response: {0}", resultGuid);
                }
            }
            catch (WebException ex)
            {
                if ((ex.Response != null) && (ex.Response is HttpWebResponse))
                {
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    Console.WriteLine("Service Error: {0} {1} ", response.StatusCode, response.StatusDescription);
                }
                else
                {
                    Console.WriteLine("Error: {0}", ex.Message);
                }
            }
            finally
            {
                sw.Stop();

                Console.WriteLine("Export Submit Complete.  Total time: {0}", sw.Elapsed.ToString());
            }
        }

        private ExportInput GetExportInput()
        {
            ExportInput ei = new ExportInput();
            ei.BlobCredentials = Program.GetBlobCredentials(this.blobUrl, this.blobAccessKey, this.blobAccessKeyType);
            ei.ConnectionInfo = this.connectionInfo;
            return ei;
        }
    }
}
