﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using DacIEServiceCli.IEService;

namespace DacIEServiceCli
{
    public partial class Program
    {
        public void ImportAction()
        {
            Stopwatch sw = new Stopwatch();

            try
            {
                Console.WriteLine("Import started: {0}", DateTime.Now);
                sw.Start();

                IDACWebService channel = this.CreateWebServiceChannel();
                using (channel as IDisposable)
                {
                    var importInput = this.GetImportInput();
                    Guid resultGuid = channel.Import(importInput);
                    Console.WriteLine("Operation Response: {0}", resultGuid);
                }
            }
            catch (WebException ex)
            {
                if ((ex.Response != null) && (ex.Response is HttpWebResponse))
                {
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    Console.WriteLine("Service Error: {0} {1} ", response.StatusCode, response.StatusDescription);
                }
                else
                {
                    Console.WriteLine("Error: {0}", ex.Message);
                }
            }
            finally
            {
                sw.Stop();
                Console.WriteLine("Import Submit Complete.  Total time: {0}", sw.Elapsed.ToString());
            }
        }

        private ImportInput GetImportInput()
        {
            ImportInput ii = new ImportInput();
            BlobCredentials creds = Program.GetBlobCredentials(this.blobUrl, this.blobAccessKey, this.blobAccessKeyType);
            ii.BlobCredentials = creds;
            ii.AzureEdition = this.azureEdition;
            ii.DatabaseSizeInGB = this.azureSize;
            ii.ConnectionInfo = this.connectionInfo;
            return ii;
        }
    }
}
