﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Xml;
using DacIEServiceCli.IEService;

namespace DacIEServiceCli
{
    public partial class Program
    {
        public void StatusAction()
        {
            Stopwatch sw = new Stopwatch();

            try
            {
                Console.WriteLine(@"Status Retrieval Started: {0}", DateTime.Now);
                sw.Start();

                IDACWebService channel = this.CreateWebServiceChannel();
                using (channel as IDisposable)
                {
                    var statusInfoList = channel.GetStatus(this.connectionInfo.ServerName, this.connectionInfo.UserName, this.connectionInfo.Password, null);

                    // Make sure we show them in a sorted list
                    var sortedStats =
                        from s in statusInfoList
                        orderby s.LastModifiedTime
                        select s;

                    foreach (var statusInformation in sortedStats)
                    {
                        Console.WriteLine(@"---------------------------");
                        Console.WriteLine(@"Server: " + statusInformation.ServerName);
                        Console.WriteLine(@"Database Name: " + statusInformation.DatabaseName);
                        Console.WriteLine(@"Blob URI: " + statusInformation.BlobUri);
                        Console.WriteLine(@"Status: " + statusInformation.Status);
                        Console.WriteLine(@"Last Modified Time: " + statusInformation.LastModifiedTime);
                        Console.WriteLine(@"Request Type: " + statusInformation.RequestType);

                        if (!String.IsNullOrEmpty(statusInformation.ErrorMessage))
                        {
                            Console.WriteLine(@"Error: " + statusInformation.ErrorMessage);
                        }

                        Console.WriteLine(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                if ((ex.Response != null) && (ex.Response is HttpWebResponse))
                {
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    Console.WriteLine("Service Error: {0}\r\n{1} ", response.StatusCode, response.StatusDescription);
                }
                else
                {
                    Console.WriteLine("Error: {0}\r\n", ex.Message);
                }
            }
            finally
            {
                sw.Stop();
                Console.WriteLine(@"Status Retrieval Complete.  Total time: {0}", sw.Elapsed);
            }
        }
    }
}
