﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Data.SqlClient;
using Microsoft.Synchronization.Data.SqlServer;
using Microsoft.Synchronization.Data;
using Microsoft.Synchronization;

namespace SQLAzureDataSync
{
    class Program
    {
        public static string sqlazureConnectionString = "Server=nda8gm9i7v.database.windows.net;Database=Test;User ID=bobb@nda8gm9i7v;Password=Rmb0853!;Trusted_Connection=False;Encrypt=True;";
        public static string sqllocalConnectionString = "Server=(local);Database=AdventureWorksLT2008R2;Trusted_Connection=True";
        public static readonly string scopeName = "alltablesyncgroup";

        static void Main(string[] args)
        {
            // Test if input arguments were supplied:
            if (args.Length == 0)
            {
                System.Console.WriteLine("Please enter an argument.");
                System.Console.WriteLine("Usage: SyncTest.exe -setup");
                System.Console.WriteLine("       SyncTest.exe -sync");
            }
            else if (args[0] == "-setup")
                Setup();
            else if (args[0] == "-sync")
                Sync();
        }

        public static void Setup()
        {
            try
            {

                SqlConnection sqlServerConn = new SqlConnection(sqllocalConnectionString);
                SqlConnection sqlAzureConn = new SqlConnection(sqlazureConnectionString);
                DbSyncScopeDescription myScope = new DbSyncScopeDescription(scopeName);

                DbSyncTableDescription Customer = SqlSyncDescriptionBuilder.GetDescriptionForTable("Customer", sqlServerConn);
                DbSyncTableDescription Product = SqlSyncDescriptionBuilder.GetDescriptionForTable("Product", sqlServerConn);

                // Add the tables from above to the scope
                myScope.Tables.Add(Customer);
                myScope.Tables.Add(Product);

                // Setup SQL Server for sync
                SqlSyncScopeProvisioning sqlServerProv = new SqlSyncScopeProvisioning(sqlServerConn, myScope);
                if (!sqlServerProv.ScopeExists(scopeName))
                {
                    // Apply the scope provisioning.
                    Console.WriteLine("Provisioning SQL Server for sync " + DateTime.Now);
                    sqlServerProv.Apply();
                    Console.WriteLine("Done Provisioning SQL Server for sync " + DateTime.Now);
                }
                else
                    Console.WriteLine("SQL Server Database server already provisioned for sync " + DateTime.Now);

                // Setup SQL Azure for sync
                SqlSyncScopeProvisioning sqlAzureProv = new SqlSyncScopeProvisioning(sqlAzureConn, myScope);
                if (!sqlAzureProv.ScopeExists(scopeName))
                {
                    // Apply the scope provisioning.
                    Console.WriteLine("Provisioning SQL Azure for sync " + DateTime.Now);
                    sqlAzureProv.Apply();
                    Console.WriteLine("Done Provisioning SQL Azure for sync " + DateTime.Now);
                }
                else
                    Console.WriteLine("SQL Azure Database server already provisioned for sync " + DateTime.Now);

                sqlAzureConn.Close();
                sqlServerConn.Close();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        public static void Sync()
        {
            try
            {

                SqlConnection sqlServerConn = new SqlConnection(sqllocalConnectionString);
                SqlConnection sqlAzureConn = new SqlConnection(sqlazureConnectionString);
                SyncOrchestrator orch = new SyncOrchestrator
                {
                    LocalProvider = new SqlSyncProvider(scopeName, sqlAzureConn),
                    RemoteProvider = new SqlSyncProvider(scopeName, sqlServerConn),
                    Direction = SyncDirectionOrder.UploadAndDownload
                };
                Console.WriteLine("ScopeName={0} ", scopeName.ToUpper());
                Console.WriteLine("Starting Sync " + DateTime.Now);
                ShowStatistics(orch.Synchronize());

                sqlAzureConn.Close();
                sqlServerConn.Close();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }

        public static void ShowStatistics(SyncOperationStatistics syncStats)
        {
            string message;

            message = "\tSync Start Time :" + syncStats.SyncStartTime.ToString();
            Console.WriteLine(message);
            message = "\tSync End Time   :" + syncStats.SyncEndTime.ToString();
            Console.WriteLine(message);
            message = "\tUpload Changes Applied :" + syncStats.UploadChangesApplied.ToString();
            Console.WriteLine(message);
            message = "\tUpload Changes Failed  :" + syncStats.UploadChangesFailed.ToString();
            Console.WriteLine(message);
            message = "\tUpload Changes Total   :" + syncStats.UploadChangesTotal.ToString();
            Console.WriteLine(message);
            message = "\tDownload Changes Applied :" + syncStats.DownloadChangesApplied.ToString();
            Console.WriteLine(message);
            message = "\tDownload Changes Failed  :" + syncStats.DownloadChangesFailed.ToString();
            Console.WriteLine(message);
            message = "\tDownload Changes Total   :" + syncStats.DownloadChangesTotal.ToString();
            Console.WriteLine(message);
        }
    }
}


